<?php

namespace Lc\SovBundle\Controller\Admin;

use EasyCorp\Bundle\EasyAdminBundle\Collection\FieldCollection;
use EasyCorp\Bundle\EasyAdminBundle\Config\Action;
use EasyCorp\Bundle\EasyAdminBundle\Config\Actions;
use EasyCorp\Bundle\EasyAdminBundle\Config\Assets;
use EasyCorp\Bundle\EasyAdminBundle\Config\Crud;
use EasyCorp\Bundle\EasyAdminBundle\Config\KeyValueStore;
use EasyCorp\Bundle\EasyAdminBundle\Config\Option\EA;
use EasyCorp\Bundle\EasyAdminBundle\Context\AdminContext;
use EasyCorp\Bundle\EasyAdminBundle\Controller\AbstractCrudController as EaAbstractCrudController;
use EasyCorp\Bundle\EasyAdminBundle\Event\AfterCrudActionEvent;
use EasyCorp\Bundle\EasyAdminBundle\Event\AfterEntityUpdatedEvent;
use EasyCorp\Bundle\EasyAdminBundle\Event\BeforeCrudActionEvent;
use EasyCorp\Bundle\EasyAdminBundle\Event\BeforeEntityDeletedEvent;
use EasyCorp\Bundle\EasyAdminBundle\Event\BeforeEntityUpdatedEvent;
use EasyCorp\Bundle\EasyAdminBundle\Exception\ForbiddenActionException;
use EasyCorp\Bundle\EasyAdminBundle\Exception\InsufficientEntityPermissionException;
use EasyCorp\Bundle\EasyAdminBundle\Factory\EntityFactory;
use EasyCorp\Bundle\EasyAdminBundle\Factory\FilterFactory;
use EasyCorp\Bundle\EasyAdminBundle\Factory\PaginatorFactory;
use EasyCorp\Bundle\EasyAdminBundle\Field\FormField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextareaField;
use EasyCorp\Bundle\EasyAdminBundle\Field\TextField;
use EasyCorp\Bundle\EasyAdminBundle\Router\AdminUrlGenerator;
use EasyCorp\Bundle\EasyAdminBundle\Security\Permission;
use Lc\SovBundle\Doctrine\EntityManager;
use Lc\SovBundle\Doctrine\Extension\DevAliasInterface;
use Lc\SovBundle\Doctrine\Extension\SeoInterface;
use Lc\SovBundle\Doctrine\Extension\SortableInterface;
use Lc\SovBundle\Doctrine\Extension\TranslatableInterface;
use Lc\SovBundle\Field\CollectionField;
use Lc\SovBundle\Field\GalleryManagerField;
use Lc\SovBundle\Form\Type\Crud\PositionType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Twig\Environment;

abstract class AbstractCrudController extends EaAbstractCrudController
{
    protected $session;
    protected $request;

    public function __construct(SessionInterface $session, RequestStack $request, EntityManager $em, Environment $twig)
    {
        $this->session = $session;
        $this->request = $request;
        $this->em = $em;
        $this->twig = $twig;
    }

    public function configureActions(Actions $actions): Actions
    {
        /* Translatable */
        if (in_array(TranslatableInterface::class, class_implements($this->getEntityFqcn()))) {
            $actions->update(
                    Crud::PAGE_INDEX,
                    Action::EDIT,
                    function (Action $action) {
                        $action->setTemplatePath('@LcSov/adminlte/crud/action/translatable.html.twig');

                        return $action;
                    }
            );
        }

        /* Boutons des actions dans les listes */
        $actionsArray[Crud::PAGE_INDEX] = [
                Action::NEW => [
                        'icon' => 'plus',
                        'label' => 'Créer',
                ],
                Action::EDIT => [
                        'class' => 'btn btn-sm btn-primary',
                        'icon' => 'edit',
                        'label' => false,
                ],
                Action::DELETE => [
                        'icon' => 'trash',
                        'dropdown' => true,
                ],
                Action::BATCH_DELETE => [
                        'class' => 'btn btn-danger',
                        'icon' => 'trash',
                ],
        ];

        /* Boutons des actions dans l'édition */

        $actionSaveAndReturn = [
                'add-class' => 'float-right',
                'icon' => 'check',
        ];
        $actionIndex = [
                'icon' => 'chevron-left',
                'class' => 'btn btn-link',
        ];

        $actionsArray[Crud::PAGE_EDIT] = [
                Action::SAVE_AND_CONTINUE => [
                        'class' => 'btn btn-info float-right',
                ],
                Action::DELETE => [
                        'icon' => 'trash',
                        'class' => 'btn btn-outline-danger action-delete',
                ],
                Action::SAVE_AND_RETURN => $actionSaveAndReturn,
                Action::INDEX => $actionIndex,
        ];

        $actionsArray[Crud::PAGE_NEW] = [
                Action::SAVE_AND_ADD_ANOTHER => [
                        'class' => 'btn btn-info float-right',
                ],
                Action::SAVE_AND_RETURN => $actionSaveAndReturn,
                Action::INDEX => $actionIndex,
        ];

        $actions->add(Crud::PAGE_EDIT, Action::INDEX);
        $actions->add(Crud::PAGE_EDIT, Action::DELETE);
        $actions->add(Crud::PAGE_NEW, Action::INDEX);

        if (in_array(SortableInterface::class, class_implements($this->getEntityFqcn()))) {
            $sortAction = Action::new('sort', 'Ordonner', 'fa fa-sort')
                    ->linkToCrudAction('sort')
                    ->setCssClass('btn btn-info')
                    ->createAsGlobalAction();

            $actions->add(Crud::PAGE_INDEX, $sortAction);
        }
        $actions->reorder(Crud::PAGE_EDIT, [Action::INDEX, Action::SAVE_AND_RETURN, Action::SAVE_AND_CONTINUE]);
        $actions->reorder(Crud::PAGE_NEW, [Action::INDEX, Action::SAVE_AND_RETURN, Action::SAVE_AND_ADD_ANOTHER]);


        foreach ($actionsArray as $crudActionName => $actionsStyle) {
            foreach ($actionsStyle as $actionName => $button) {
                $actions->update(
                        $crudActionName,
                        $actionName,
                        function (Action $action) use ($button) {
                            if (isset($button['add-class'])) {
                                $action->addCssClass($button['add-class']);
                            }

                            if (isset($button['class'])) {
                                $action->setCssClass($button['class']);
                            }

                            if (isset($button['icon'])) {
                                $action->setIcon('fa fa-' . $button['icon']);
                            }

                            if (isset($button['label'])) {
                                $action->setLabel($button['label']);
                            }

                            if (isset($button['dropdown']) && $button['dropdown']) {
                                $action->addCssClass('in-dropdown');
                            }

                            return $action;
                        }
                );
            }
        }

        return $actions;
    }

    public function configureResponseParameters(KeyValueStore $responseParameters): KeyValueStore
    {
        if (Crud::PAGE_INDEX === $responseParameters->get('pageName')) {
            $responseParameters->set('fields', $this->configureFields('index'));
        }

        return $responseParameters;
    }

    public function configureCrud(Crud $crud): Crud
    {
        $crud = parent::configureCrud($crud);

        $this->setMaxResults($crud);

        return $crud;
    }

    public function setMaxResults(Crud $crud)
    {
        $entityClass = $this->getEntityFqcn();
        $paramListMaxResults = 'listMaxResults';
        $paramSessionListMaxResults = $entityClass . '-' . $paramListMaxResults;
        $requestListMaxResults = $this->request->getCurrentRequest()->get($paramListMaxResults);

        if ($requestListMaxResults) {
            $this->session->set($paramSessionListMaxResults, $requestListMaxResults);
        }
        $maxResults = $this->session->get($paramSessionListMaxResults) ? $this->session->get(
                $paramSessionListMaxResults
        ) : 30;

        $crud->setPaginatorPageSize($maxResults);
    }

    public function configureFields(string $pageName): iterable
    {
        if (in_array(SeoInterface::class, class_implements($this->getEntityFqcn()))) {
            $seoPanel = [
                    FormField::addPanel('Seo'),
                    TextField::new('metaTitle')->setLabel('Meta Title')->setHelp(
                            'Affiché dans les résultats de recherche Google'
                    )->hideOnIndex(),
                    TextareaField::new('metaDescription')->setLabel('Meta description')->setHelp(
                            'Affiché dans les résultats de recherche Google'
                    )->hideOnIndex(),
                    CollectionField::new('oldUrls')
                            ->setFormTypeOption('entry_type', TextType::class)->setLabel(
                                    'Anciennes urls du document'
                            )->hideOnIndex(),
            ];
        }

        if (in_array(DevAliasInterface::class, class_implements($this->getEntityFqcn()))) {
            $confPanel = [
                    FormField::addPanel('Conf'),
                    TextField::new('devAlias')->hideOnIndex(),
            ];
        }

        return array_merge($seoPanel, $confPanel);
    }



    public function sort(AdminContext $context)
    {
        $event = new BeforeCrudActionEvent($context);
        $this->get('event_dispatcher')->dispatch($event);
        if ($event->isPropagationStopped()) {
            return $event->getResponse();
        }

        if (!$this->isGranted(Permission::EA_EXECUTE_ACTION) || !in_array(
                        SortableInterface::class,
                        class_implements($this->getEntityFqcn())
                )) {
            throw new ForbiddenActionException($context);
        }

        $fields = FieldCollection::new($this->configureFields(Crud::PAGE_INDEX));
        $filters = $this->get(FilterFactory::class)->create(
                $context->getCrud()->getFiltersConfig(),
                $fields,
                $context->getEntity()
        );
        $queryBuilder = $this->createIndexQueryBuilder($context->getSearch(), $context->getEntity(), $fields, $filters);
        $paginator = $this->get(PaginatorFactory::class)->create($queryBuilder);

        $entities = $this->get(EntityFactory::class)->createCollection($context->getEntity(), $paginator->getResults());
        $this->get(EntityFactory::class)->processFieldsForAll($entities, $fields);

        $sortableForm = $this->createFormBuilder(array('entities', $paginator->getResults()))
                ->add(
                        'entities',
                        CollectionType::class,
                        array(
                                'required' => true,
                                'allow_add' => true,
                                'entry_type' => PositionType::class,
                        )
                )
                ->getForm();

        $entityManager = $this->getDoctrine()->getManagerForClass($this->getEntityFqcn());
        $repository = $entityManager->getRepository($this->getEntityFqcn());

        $sortableForm->handleRequest($context->getRequest());

        if ($sortableForm->isSubmitted() && $sortableForm->isValid()) {
//            $entityInstance = $event->getEntityInstance();

            foreach ($sortableForm->get('entities')->getData() as $elm) {
                $entityInstance = $repository->find($elm['id']);
                $entityDto = $context->getEntity()->newWithInstance($entityInstance);

                if (!$entityDto->isAccessible()) {
                    throw new InsufficientEntityPermissionException($context);
                }

                $event = new BeforeEntityDeletedEvent($entityInstance);
                $this->get('event_dispatcher')->dispatch($event);
                $entityInstance = $event->getEntityInstance();

                $entityInstance->setPosition($elm['position']);
                $this->updateEntity($entityManager, $entityInstance);

                $this->get('event_dispatcher')->dispatch(new AfterEntityUpdatedEvent($entityInstance));
            }

            $url = $this->get(AdminUrlGenerator::class)
                    ->setAction(Action::INDEX)
                    ->generateUrl();
            $this->addFlash('success', 'Position modifié', array(), 'mweb');

            return $this->redirect($url);
        }

        $responseParameters = $this->configureResponseParameters(
                KeyValueStore::new(
                        [
                                'pageName' => Crud::PAGE_INDEX,
                                'templatePath' => '@LcSov/adminlte/crud/sort.html.twig',
                                'entities' => $entities,
                                'paginator' => $paginator,
                                'global_actions' => array(),
                                'batch_actions' => array(),
                                'filters' => $filters,
                                'sortable_form' => $sortableForm,
                        ]
                )
        );
        $responseParameters->set('fields', $this->configureFields('index'));
        $event = new AfterCrudActionEvent($context, $responseParameters);
        $this->get('event_dispatcher')->dispatch($event);
        if ($event->isPropagationStopped()) {
            return $event->getResponse();
        }

        return $responseParameters;
    }
}

